import _to_parent
from concurrent.futures import ThreadPoolExecutor
from libraries.subnet_scan import SubnetScanner
from libraries.net_tools import Device
from time import time


network = '10.0.0.0/20'

def test_ping_reliability(threads=128,scans=2):
    scanner = SubnetScanner(network, 'small', 1)
    def scan_host():
        count = 0
        with ThreadPoolExecutor(max_workers=threads) as executor:
            futures = {executor.submit(scanner._ping, Device(str(ip))): str(ip) for ip in scanner.subnet}
            for future in futures:
                ans = future.result()
                if ans:
                    count += 1
        return count
    scans = [scan_host() for _ in range(scans)]
    return scans
    
for i in range(20):
    start = time()
    threads = (i+1)*50
    scans = test_ping_reliability(threads=threads) 
    t = round(time()-start)
    print(f'Threads: {threads} Cnt:  Results: {scans} Time: {t}s')


import json
from .resource_manager import ResourceManager

DB = json.loads(ResourceManager('mac_addresses').get('mac_db.json'))


def lookup_mac(mac: str) -> str:
    """
    Lookup a MAC address in the database and return the vendor name.
    """
    if mac:
        for m in DB:
            if mac.upper().startswith(str(m).upper()):
                return DB[m]
    return None
        


from flask import request, jsonify
from . import api_bp
from ...libraries.port_manager import PortManager

# Port Manager API
############################################
@api_bp.route('/api/port/list', methods=['GET'])
def get_port_lists():
    return jsonify(PortManager().get_port_lists())

@api_bp.route('/api/port/list/<port_list>', methods=['GET'])
def get_port_list(port_list):
    return jsonify(PortManager().get_port_list(port_list))

@api_bp.route('/api/port/list/<port_list>', methods=['POST'])
def create_port_list(port_list):
    data = request.get_json()
    return jsonify(PortManager().create_port_list(port_list, data))

@api_bp.route('/api/port/list/<port_list>', methods=['PUT'])
def update_port_list(port_list):
    data = request.get_json()
    return jsonify(PortManager().update_port_list(port_list, data))

@api_bp.route('/api/port/list/<port_list>', methods=['DELETE'])
def delete_port_list(port_list):
    return jsonify(PortManager().delete_port_list(port_list))



import logging
from logging.handlers import RotatingFileHandler


def configure_logging(loglevel:str, logfile:bool):
    numeric_level = getattr(logging, loglevel.upper(), None)
    if not isinstance(numeric_level, int):
        raise ValueError(f'Invalid log level: {loglevel}')
    
    
    logging.basicConfig(level=numeric_level, format='[%(name)s] %(levelname)s - %(message)s')
    
    if logfile:
        handler = RotatingFileHandler('lanscape.log', maxBytes=100000, backupCount=3)
        handler.setLevel(numeric_level)
        formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
        handler.setFormatter(formatter)
        logging.getLogger().addHandler(handler)
    else:
        # For console, it defaults to basicConfig
        pass
    


import _to_parent
import unittest
import json
from app import app
from libraries.net_tools import get_primary_network_subnet
from libraries.subnet_scan import cleanup_old_jobs




class AppTestCase(unittest.TestCase):
    app = app.test_client()

    def test_port_lifecycle(self):
        # Delete the new port list if it exists
        self.app.delete('/api/port/list/test_port_list_lifecycle')

        # Get the list of port lists
        response = self.app.get('/api/port/list')
        self.assertEqual(response.status_code, 200)
        port_list_start = json.loads(response.data)

        # Create a new port list
        new_port_list = {'80': 'http', '443': 'https'}
        response = self.app.post('/api/port/list/test_port_list_lifecycle', json=new_port_list)
        self.assertEqual(response.status_code, 200)


        # Get the list of port lists again
        response = self.app.get('/api/port/list')
        self.assertEqual(response.status_code, 200)
        port_list_new = json.loads(response.data)
        # Verify that the new port list is in the list of port lists
        self.assertEqual(len(port_list_new), len(port_list_start) + 1)

        # Get the new port list
        response = self.app.get('/api/port/list/test_port_list_lifecycle')
        self.assertEqual(response.status_code, 200)
        port_list = json.loads(response.data)
        self.assertEqual(port_list, new_port_list)

        # Update the new port list
        updated_port_list = {'22': 'ssh', '8080': 'http-alt'}
        response = self.app.put('/api/port/list/test_port_list_lifecycle', json=updated_port_list)
        self.assertEqual(response.status_code, 200)

        # Get the new port list again
        response = self.app.get('/api/port/list/test_port_list_lifecycle')
        self.assertEqual(response.status_code, 200)
        port_list = json.loads(response.data)

        # Verify that the new port list has been updated
        self.assertEqual(port_list, updated_port_list)

        # Delete the new port list
        response = self.app.delete('/api/port/list/test_port_list_lifecycle')
        self.assertEqual(response.status_code, 200)

    def test_scan(self):
        # Delete the new port list if it exists
        self.app.delete('/api/port/list/test_port_list_scan')

        # Create a new port list
        new_port_list = {'80': 'http', '443': 'https'}
        response = self.app.post('/api/port/list/test_port_list_scan', json=new_port_list)
        self.assertEqual(response.status_code, 200)

        # Create a new scan
        new_scan = {'subnet': get_primary_network_subnet(), 'port_list': 'test_port_list_scan'}
        response = self.app.post('/api/scan/async', json=new_scan)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(json.loads(response.data)['status'], 'complete')

        # Delete the new port list
        response = self.app.delete('/api/port/list/test_port_list_scan')
        self.assertEqual(response.status_code, 200)

        cleanup_old_jobs(True)

        
        





if __name__ == '__main__':
    unittest.main()


from .webviewer import start_webview
from .app import start_webserver
import threading
import webbrowser
import argparse
import time
import logging
import traceback
from .libraries.logger import configure_logging

log = logging.getLogger('core')


def main():
    args = parse_args()
    configure_logging(args.loglevel, args.logfile)
    def no_gui():
        open_browser(f'http://127.0.0.1:{args.port}')
        start_webserver(
            debug=args.debug,
            port=args.port
        )
        
        
        
    try:
        if args.nogui:
            no_gui()
        else:
            start_webview(
                port=args.port
            )
    except Exception:
        # showing error in debug only because this is handled gracefully
        log.debug('Failed to start webview client. Traceback below')
        log.debug(traceback.format_exc())
        log.error('Unable to start webview client. Try running with flag --nogui')
        

def parse_args():
    parser = argparse.ArgumentParser(description='LANscape')
    parser.add_argument('--debug', action='store_true', help='Run in debug mode')
    parser.add_argument('--port', type=int, default=5001, help='Port to run the webserver on')
    parser.add_argument('--nogui', action='store_true', help='Run in standalone mode')
    parser.add_argument('--logfile', action='store_true', help='Log output to lanscape.log')
    parser.add_argument('--loglevel', default='INFO', choices=['DEBUG', 'INFO', 'WARNING', 'ERROR', 'CRITICAL'], help='Set the log level')

    return parser.parse_args()

def open_browser(url: str,wait=2):
    """
    Open a browser window to the specified
    url after waiting for the server to start
    """
    def do_open():
        time.sleep(wait)
        webbrowser.open(url, new=2)

    threading.Thread(target=do_open).start()



if __name__ == "__main__":
    main()
        
